
/*
 * Still Grab 0.1
 * 6/21/97
 * 
 * Danny Dulai <nirva@ishiboo.com>
 * 
 */
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <termios.h>

#ifdef SONY_DSCF1
#  define CAMERANAME "Sony DSC-F1"
#  include "dscf1.h"
#else
#  ifdef CASIO_QV10 
#    define CAMERANAME "Casio QV-10"
#    include "qv10.h"
#  endif
#endif

#include "main.h"
#include "tty.h"
#include "gfx.h"

extern char *optarg;
int getopt();

char		*filename = "grab";
char		*progname;
u_char		*imagedata;
int		totalpics;
Thumbnail	*thumbnails;

int		onpic=0;

int
usage()
{
    printf("\n"
	   "StillGrab CLI 1.0 -- %s Frame grabber\n"
	   "Usage: %s [-f filename] [-h]\n"
	   "\n"
	   "-d              Delete pictures instead of retrieving them\n"
	   "-h              This help\n"
	   "-f filename     Default filename prefix to save grabs\n"
	   "-s #            Picture number to start at (default 0)\n"
	   "-e #            Picture number to end at (default last)\n"
	   "-T              Path to serial device\n"
	   "-S [h|l]        Speed: h = 38400bps, l = 9600bps\n"
	   "\n" , CAMERANAME, progname);
	return 0;
}

void
Exit(int sig)
{
    if(sig)
	printf("Caught signal %d\n",sig);

    CloseCamera();
    
    printf("Quitting...\n");
    exit(0);
}

FILE *
OpenGrabFile()
{
    FILE *fp;
    char name[1024];
    int fd;
    int i = 1000;
    static int savepicnum=0;

    while(i--) {
	strncpy(name, filename, 1000);
	sprintf(name+strlen(name), "%d.jpg", savepicnum++);
	if ((fd = open(name, O_CREAT|O_EXCL|O_WRONLY, 0644)) >= 0) {
	    fp = fdopen(fd, "w");
	    return fp;
	}
    }
    printf("couldn't open grab file\n");
    Exit(0);
    return NULL;
}

void
main(int argc, char *argv[])
{
    char  *devpath = RSPORT;
    int ch, i, speed = DEFAULTBAUD, startpic, endpic, endpictmp, deleteing;
    
    progname = argv[0];

    signal(SIGBUS, Exit);
    signal(SIGINT, Exit);
    signal(SIGHUP, Exit);
    signal(SIGQUIT, Exit);
    signal(SIGSEGV, Exit);
    signal(SIGTERM, Exit);
    
    startpic = 1;
    endpictmp = -1;
    deleteing = 0;

    while ((ch = getopt(argc, argv, "s:e:T:f:S:dh")) != EOF)
	switch(ch) {
	    case 'S':
		switch(optarg[0]){
		    case 'h':
			speed = B38400;
			break;
		    case 'l':
			speed = B9600;
			break;
		}
		break;
	    case 'e':
		endpictmp = atoi(optarg);
		break;
	    case 's':
		startpic = atoi(optarg);
		break;
	    case 'd':
		deleteing = 1;
		break;
	    case 'T':
		devpath = optarg;
		break;
	    case 'f':
		filename = optarg;
		break;
	    case '?':
	    case 'h':
	    default:
		usage();
		exit(0);
	}

    if (InitCamera(devpath, speed) < 0) {
      	printf("Camera not responding\n");
	exit(1);
    }

    totalpics = GetPicCount();
    
    if (startpic > totalpics || startpic < 1) {
	printf("Invalid start picture: %d. Total of %d pictures.\n", startpic, totalpics);
	exit(1);
    }

    if (endpictmp != -1) {
	endpic = endpictmp-1;
	if (endpictmp > totalpics || endpictmp < 1) {
	    printf("Invalid end picture: %d. Total of %d pictures.\n", endpictmp, totalpics);
	    exit(1);
	}
    } else
	endpic = totalpics-1;

    startpic--;
    
    if (startpic > endpic) {
	int t = endpic;
	endpic = startpic;
	startpic = t;
	printf("Swapping start and end pictures.\n");
    }

    printf("retrieving pictures %d - %d out of total %d\n", startpic+1, endpic+1, totalpics);
    InitThumbnails(totalpics);

    for (onpic = startpic; onpic <= endpic; ) {
	if (deleteing) {
	    printf("Removing picture %d.\n", onpic+1);
	    RemoveImage(onpic++);
	} else {
	    int size=1, notdone = 1;
	    while(notdone) {
		switch((i=GetPicture())) {
		    case -1:
			printf("F1finfo() failure\n");
			break;
		    case -2:
			printf("F1fopen() failure\n");
			break;
		    case -3:
			printf("F1fread() failure\n");
			break;
		    case 0:
			notdone = 0;
			printf("100%% done\n");
			break;
		    default:
			if(i >= 2) {
			    size = i-2;
			    printf("picture %d, %d bytes - ", onpic+1, size);
			} else if(i <= -3) {
			    printf("%3.3d%% done\b\b\b\b\b\b\b\b\b", (-(i+3))*100/size);
			    fflush(stdout);
			}
			break;
		}
	    }
	}
    }
    Exit(0);
}
