#ifdef SONY_DSCF1

#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <termios.h>
#include <setjmp.h>
#include <X11/xpm.h>
#include "main.h"
#include "tty.h"
#include "dscf1_low.h"
#include "jpeglib.h"

#define PMP_RESOLUTION 0x1d
#define PMP_FIN 0x33
#define PMP_STD 0x17
#define PMP_ECM 0x08

#define PMP_COMMENT 0x34

#define PMP_TAKE_YEAR 0x4c
#define PMP_TAKE_MONTH 0x4d
#define PMP_TAKE_DATE 0x4e
#define PMP_TAKE_HOUR 0x4f
#define PMP_TAKE_MINUTE 0x50
#define PMP_TAKE_SECOND 0x51

#define PMP_EDIT_YEAR 0x54
#define PMP_EDIT_MONTH 0x55
#define PMP_EDIT_DATE 0x56
#define PMP_EDIT_HOUR 0x57
#define PMP_EDIT_MINUTE 0x58
#define PMP_EDIT_SECOND 0x59

#define PMP_SPEED 0x66

#define PMP_FLASH 0x76

static int totalbytes=0;

void jpeg_buf_src(j_decompress_ptr cinfo, u_char *buf, size_t buflen);

struct my_error_mgr {
    struct jpeg_error_mgr pub;	/* "public" fields */
    jmp_buf setjmp_buffer;	/* for return to caller */
};

typedef struct my_error_mgr * my_error_ptr;

void
my_error_exit(j_common_ptr cinfo)
{
    my_error_ptr myerr = (my_error_ptr) cinfo->err;
    (*cinfo->err->output_message) (cinfo);
    longjmp(myerr->setjmp_buffer, 1);
}

#ifndef CLI
int 
Data2XImage(char *buf, int buflen, XImage *xim, int flip)
{
    struct jpeg_decompress_struct cinfo;
    struct my_error_mgr jerr;
    JSAMPARRAY buffer;		/* Output row buffer */
    int row_stride;		/* physical row width in output buffer */
    char *ximdata, *ximdata2;

    /* We set up the normal JPEG error routines, then override error_exit. */
    cinfo.err = jpeg_std_error(&jerr.pub);
    jerr.pub.error_exit = my_error_exit;

    /* Establish the setjmp return context for my_error_exit to use. */
    if (setjmp(jerr.setjmp_buffer)) {
	jpeg_destroy_decompress(&cinfo);
	return 1;
    }

    jpeg_create_decompress(&cinfo);
    jpeg_buf_src(&cinfo, buf, buflen);
    jpeg_read_header(&cinfo, TRUE);
    jpeg_start_decompress(&cinfo);
    row_stride = cinfo.output_width * cinfo.output_components;
    buffer = (*cinfo.mem->alloc_sarray)((j_common_ptr) &cinfo, JPOOL_IMAGE, row_stride, 1);

    if(flip)
	ximdata2=ximdata=xim->data+(bperline*63);
    else
	ximdata2=ximdata=xim->data;

    while (cinfo.output_scanline < cinfo.output_height) {
	int i;
	u_long r,g,b, xcol;
	jpeg_read_scanlines(&cinfo, buffer, 1);

	/* Assume put_scanline_someplace wants a pointer and sample count. */
	/* put_scanline_someplace(buffer[0], row_stride); */    

	ximdata = ximdata2;
	if(flip)
	    ximdata2 -= bperline;
	else
	    ximdata2 += bperline;

	for(i=0; i < row_stride; i+=3) {
	    if(flip) {
		r = buffer[0][row_stride-i-3];
		g = buffer[0][row_stride-i-2];
		b = buffer[0][row_stride-i-1];
	    } else {
		r = buffer[0][i+0];
		g = buffer[0][i+1];
		b = buffer[0][i+2];
	    }

	    /* shift the bits around */
	    if (rshift<0) r = r << (-rshift);
	    else r = r >> rshift;

	    if (gshift<0) g = g << (-gshift);
	    else g = g >> gshift;

	    if (bshift<0) b = b << (-bshift);
	    else b = b >> bshift;

	    r = r & rmask;
	    g = g & gmask;
	    b = b & bmask;

	    xcol = r | g | b;         
	    if (bperpix == 32) {
		if (border == MSBFirst) {
		    *ximdata++ = (xcol>>24) & 0xff;
		    *ximdata++ = (xcol>>16) & 0xff;
		    *ximdata++ = (xcol>>8)  & 0xff;
		    *ximdata++ =  xcol      & 0xff;
		} else {  /* LSBFirst */
		    *ximdata++ =  xcol      & 0xff;
		    *ximdata++ = (xcol>>8)  & 0xff;
		    *ximdata++ = (xcol>>16) & 0xff;
		    *ximdata++ = (xcol>>24) & 0xff;
		}
	    }
	}
    }

    jpeg_finish_decompress(&cinfo);
    jpeg_destroy_decompress(&cinfo);

    return 0;
}
#endif

int
make_jpeg_comment(u_char *buf, u_char *jpeg_comment)
{
    int i, cur = 0;
    int reso, shutter;

    struct resolution {
	int reso_val;
	char *reso_conv;
    } reso_tab[] = {
	{PMP_FIN, "fine"},
	{PMP_STD, "standard"},
	{PMP_ECM, "economy"},
	{0,       "unknown"},
    }; 

    struct sh_speed {
	int spd_val;
	char *spd_conv;
    } sh_speed_tab[] = {
	{0x0123, "1/7.5"},
	{0x0187, "1/15"},
	{0x01eb, "1/30"},
	{0x024f, "1/60"},
	{0x0298, "1/100"},
	{0x031d, "1/250"},
	{0x0381, "1/500"},
	{0x03e5, "1/1000"},
	{0,      "unknown"},
    };

    jpeg_comment[0] = 0xff;
    jpeg_comment[1] = 0xd8;
    jpeg_comment[2] = 0xff;
    jpeg_comment[3] = 0xfe;

    /* resolution */
    reso = *(buf+PMP_RESOLUTION);

    for (i=0; ;i++)
	if ((reso == reso_tab[i].reso_val) || (reso_tab[i].reso_val == 0)) {
	    cur = 6 + sprintf(&jpeg_comment[6], "Resolution: %s\n", reso_tab[i].reso_conv);
	    break;
	}

    /* shutter speed */
    shutter = (int)(buf[PMP_SPEED] << 8) | buf[PMP_SPEED+1];

    for (i=0; ;i++)
	if ((shutter == sh_speed_tab[i].spd_val) || (sh_speed_tab[i].spd_val == 0)) {
	    cur += sprintf(&jpeg_comment[cur], "Shutter-speed: %s\n", sh_speed_tab[i].spd_conv);
	    break;
	}

    /* PMP comment */
    if (*(buf+PMP_COMMENT))
	cur += sprintf(&jpeg_comment[cur], "Comment: %s\n", (char *)(buf+PMP_COMMENT));

    /* taken date */
    if (*(buf+PMP_TAKE_YEAR) == 0xff)
	cur += sprintf(&jpeg_comment[cur], "Date-Taken: ----/--/-- --:--:--\n");

    else {
	cur += sprintf(&jpeg_comment[cur],
			    "Date-Taken: %d/%02d/%02d %02d:%02d:%02d\n",
			    1900+(*(buf+PMP_TAKE_YEAR)), *(buf+PMP_TAKE_MONTH),
			    *(buf+PMP_TAKE_DATE), *(buf+PMP_TAKE_HOUR), *(buf+PMP_TAKE_MINUTE),
			    *(buf+PMP_TAKE_SECOND)); 
    }

    /* edited date */
    if (*(buf+PMP_EDIT_YEAR) == 0xff)
	cur += sprintf(&jpeg_comment[cur], "Date-Edited: ----/--/-- --:--:--\n");
    else
	cur += sprintf(&jpeg_comment[cur],
			    "Date-Edited: %d/%02d/%02d %02d:%02d:%02d\n",
			    1900+(*(buf+PMP_EDIT_YEAR)), *(buf+PMP_EDIT_MONTH),
			    *(buf+PMP_EDIT_DATE), *(buf+PMP_EDIT_HOUR), *(buf+PMP_EDIT_MINUTE),
			    *(buf+PMP_EDIT_SECOND));

    /* use flash? */
    if (*(buf+PMP_FLASH) != 0)
	cur += sprintf(&jpeg_comment[cur], "Flash: on\n");

    /* insert total jpeg comment length */
    jpeg_comment[4] = (u_char)((cur - 4) >> 8);
    jpeg_comment[5] = (u_char)(cur - 4);

    return cur;
}


long
GetPicture()
{
    u_long filelen;
    static u_char buf[0x1000];
    long len;
    char name[64];
    
    if (!totalbytes) {
	sprintf(name, "/PIC_CAM/PIC00000/PSN%05d.PMP", thumbnails[onpic].pic_ind);

	CamOk();
	CamStatus(0);

	filelen = CamFinfo(name);
	if (filelen == 0)
	    return(-1);

	if (CamFopen(name) != 0)
	    return(-2);

	len = CamFread(buf, 126);
	if (len < 126) {
	    CamFclose();
	    return(0);
	}
	thumbnails[onpic].fp = OpenGrabFile();
	fwrite(buf+0x500, 1, make_jpeg_comment(buf, buf+0x500), thumbnails[onpic].fp);
	totalbytes = 126;
	return 2+filelen;
    }

    if ((len = CamFread(buf, 0x100)) != 0) {
	if (len < 0) {
	    CamFclose();
	    return(-3);
	}
	totalbytes += len;
	fwrite(buf, sizeof(u_char), (size_t) len, thumbnails[onpic].fp);
	return -3-totalbytes;
    }
    fclose(thumbnails[onpic].fp);
    CamFclose();

    totalbytes = 0;
    onpic++;

    return 0;
}

long
GetThumbnail()
{
    u_long filelen;
    static u_char buf[0x1000];
    static u_char *p;
    long len;
    int i;
    char name[64];

    if(!totalbytes) {
	int n;
	sprintf(name, "/PIC_CAM/PIC00000/PIDX%03d.PMX", (thumbnails[onpic].thumb_ind & 0xff));
	p = buf;

	CamOk();
	CamStatus(0);
	n = (thumbnails[onpic].thumb_ind >> 8) & 0xff;

	filelen = CamFinfo(name);
	if(filelen == 0)
	    return(-1);

	if(CamFopen(name) != 0)
	    return(-2);

	for( i = 0 ; i < n ; i++) 
	    len = CamFseek(0x1000 , 1);
    }

    if((len = CamFread(p, 0x100)) != 0){
	if(len < 0) {
	    CamFclose();
	    return(-3);
	}
	totalbytes += len;
	p += len;
	if(totalbytes < 0x1000)
	    return 1;
    }
    CamFclose();

    filelen = buf[12] * 0x1000000 + buf[13] * 0x10000 + buf[14] * 0x100 + buf[15];

    thumbnails[onpic].thumblen = filelen;
    thumbnails[onpic].thumb = malloc(filelen);
    memcpy(thumbnails[onpic].thumb, buf+256, filelen);

    totalbytes = 0;
    onpic++;

    return 0;
}

int
InitThumbnails(int total)
{
    u_char buf[4096];
    char name[64];
    int i, n, j, k;

    thumbnails = calloc(total, sizeof(Thumbnail));

    sprintf(name, "/PIC_CAM/PIC00000/PIC_INF.PMF");
    CamGetdata(name, buf, 0);

    n = buf[26] * 256 + buf[27];
    if(n != totalpics)
	printf("error! %d != %d\n",n, totalpics);

    for(i=k=0; i < buf[31]; i++)
	for(j = 0 ; j < buf[0x20 + 4 * i + 3]; j++)
	    thumbnails[k++].thumb_ind = (j << 8) | buf[0x20 + 4 * i] ;

    for(i=0; i<totalpics ; i++) {
	thumbnails[i].pic_ind = buf[0x420 + 0x10 * i + 3];
	thumbnails[i].rotate = buf[0x420 + 0x10 * i + 5];
	thumbnails[i].protect = buf[0x420 + 0x10 * i + 14];
    }

    return(n);
}

#endif
