#include <stdio.h>
#include <sys/types.h>
#include "jpeglib.h"
#include "jerror.h"

typedef struct {
    struct jpeg_source_mgr pub;	/* public fields */

    u_char *inbuf;
    size_t buflen;
    JOCTET * buffer;
} my_source_mgr;

typedef my_source_mgr * my_src_ptr;

#define INPUT_BUF_SIZE  4096	/* choose an efficiently fread'able size */

void
init_source(j_decompress_ptr cinfo)
{
}

boolean
fill_input_buffer(j_decompress_ptr cinfo)
{
    my_src_ptr src = (my_src_ptr) cinfo->src;
    size_t nbytes;

    if(src->buflen < INPUT_BUF_SIZE)
	nbytes = src->buflen;
    else
	nbytes = INPUT_BUF_SIZE;
    memcpy(src->buffer, src->inbuf, nbytes);
    src->inbuf += nbytes;
    src->buflen -= nbytes;

    if (nbytes <= 0) {
	WARNMS(cinfo, JWRN_JPEG_EOF);
	/* Insert a fake EOI marker */
	src->buffer[0] = (JOCTET) 0xFF;
	src->buffer[1] = (JOCTET) JPEG_EOI;
	nbytes = 2;
    }

    src->pub.next_input_byte = src->buffer;
    src->pub.bytes_in_buffer = nbytes;

    return TRUE;
}

void
skip_input_data (j_decompress_ptr cinfo, long num_bytes)
{
    my_src_ptr src = (my_src_ptr) cinfo->src;

    if (num_bytes > 0) {
	while (num_bytes > (long) src->pub.bytes_in_buffer) {
	    num_bytes -= (long) src->pub.bytes_in_buffer;
	    fill_input_buffer(cinfo);
	}
	src->pub.next_input_byte += (size_t) num_bytes;
	src->pub.bytes_in_buffer -= (size_t) num_bytes;
    }
}


void
term_source (j_decompress_ptr cinfo)
{
    /* no work necessary here */
}

void
jpeg_buf_src(j_decompress_ptr cinfo, u_char *buf, size_t buflen)
{
    my_src_ptr src;

    if (cinfo->src == NULL) {
	cinfo->src = (struct jpeg_source_mgr *) (*cinfo->mem->alloc_small) ((j_common_ptr) cinfo, JPOOL_PERMANENT, sizeof(my_source_mgr));
	src = (my_src_ptr) cinfo->src;
	src->buffer = (JOCTET *) (*cinfo->mem->alloc_small) ((j_common_ptr) cinfo, JPOOL_PERMANENT, INPUT_BUF_SIZE * sizeof(JOCTET));
    }

    src = (my_src_ptr) cinfo->src;
    src->pub.init_source = init_source;
    src->pub.fill_input_buffer = fill_input_buffer;
    src->pub.skip_input_data = skip_input_data;
    src->pub.resync_to_restart = jpeg_resync_to_restart;
    src->pub.term_source = term_source;
    src->pub.bytes_in_buffer = 0;
    src->pub.next_input_byte = NULL;
    src->inbuf = buf;
    src->buflen = buflen;
}
