#include "config.h"
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#if HAVE_FCNTL_H
# include <fcntl.h>
#endif
#include <stdlib.h>
#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif
#if HAVE_TERMIOS_H
# include <termios.h>
#else
# if HAVE_TERMIO_H
#  include <termio.h>
# else
#  if HAVE_SYS_IOCTL_H
#   include <sys/ioctl.h>
#  endif
#  include <sgtty.h>
# endif
#endif

#include "tty.h"

#define	TTYTIMEOUT	3

int	Camfd = -1;

void Exit(int sig);

int
changespeed(int fd, int baud)
{
#if HAVE_TERMIOS_H
  /* termios interface */
  struct termios tio;

  if (tcgetattr(fd, &tio) < 0) {
    perror("tcgetattr");
    close(fd);
    return(-1);
  }
  tio.c_iflag = 0;
  tio.c_oflag = 0;
  tio.c_cflag = CS8 | CREAD | CLOCAL  ; /* 8bit non parity stop 1 */
  tio.c_lflag = 0;
  tio.c_cc[VMIN] = 1;
  tio.c_cc[VTIME] = 5 ;
  cfsetispeed(&tio, baud);
  cfsetospeed(&tio, baud);
  if (tcsetattr(fd, TCSANOW, &tio) < 0) {
    perror("tcsetattr");
    return(-1);
  }
#else
# if HAVE_TERMIO_H
  /* termio interface */
  /*  #  error not implemented yet. */
  fprintf(stderr,"sorry not inplemented yet.\n");
# else
  /* sgtty interface */
  struct sgttyb ttyb;
  if (ioctl(fd, TIOCGETP, &ttyb) < 0) {
    perror("tcgetattr");
    return(-1);
  }
  
  ttyb.sg_ispeed = baud;
  ttyb.sg_ospeed = baud;
  ttyb.sg_flags = 0;		/* 8bit non parity stop 1 */
  if (ioctl(fd, TIOCSETP, &ttyb) < 0) {
    perror("tcsetattr");
    return(-1);
  }

# endif
#endif
  flushtty(fd);
  return(1);
}

int
opentty(char *path, int speed)
{
    int fd;

    if ((fd = open(path, O_RDWR | O_NDELAY )) != -1) {
#ifndef HAVE_TERMIOS_H
	if(ioctl(fd, TIOCEXCL, 0) < 0){
	    fprintf(stderr, "Can't set tty exclusive mode.\n");
	    return -1;
	}
	if(ioctl(fd, TIOCHPCL, 0) < 0){
	    fprintf(stderr, "Can't set tty hold mode.\n");
	    return -1;
	}
#endif
	if(changespeed(fd, speed) < 0)
	    return -1; 
    } else {
	fprintf(stderr, "Can't open tty.\n");
	return -1;
    }

    return fd;
}

int
readtty(int fd, u_char *p, int c)
{
    fd_set readfds;
    int nfds, i;
    struct timeval tv;
    int retry=3;

    FD_ZERO(&readfds);
    FD_SET(fd, &readfds);
    tv.tv_sec = TTYTIMEOUT;
    tv.tv_usec = 0;

    while (c)
	if (!(nfds = select(fd+1, &readfds, NULL, NULL, &tv))) {
	    fprintf(stderr,"tty not respond. time up.\n");
	    if(retry--)
		continue;
	    return -1;
	} else {
	    if(FD_ISSET(fd, &readfds)) {
		if((i = read(fd, p, c)) < 0) {
		    fprintf(stderr,"tty read fail.\n");
		    return -1;
		}
		p += i;
		c -= i;
	    }
	}
    return 0;
}

void
flushtty(int fd)
{
    u_char c[1024];
    fd_set readfds;
    int nfds;
    struct timeval tv;

    FD_ZERO(&readfds);
    FD_SET(fd, &readfds);
    tv.tv_sec = 0;
    tv.tv_usec = 0;

    while (1) {
	if (!(nfds = select(fd+1, &readfds, NULL, NULL, &tv)))
	    return;
	else {
	    if(FD_ISSET(fd, &readfds))
		if(read(fd, &c, 1024) < 0){
		    perror("read(tty)");
		    return;
		}
	}
    }
}
void
Setfd(int fd)
{
    Camfd = fd;
}

int
Getfd()
{
    return Camfd;
}

void
wbyte(u_char c)
{
    if (writetty(Camfd, &c, 1) < 0) {
	perror("writetty");
	Exit(1);
    }
}

u_char
rbyte()
{
    u_char c;

    if (readtty(Camfd, &c, 1) < 0) {
	perror("readtty");
	Exit(1);
    }
    return c;
}

inline void
wstr(u_char *p, int len)
{
    if (writetty(Camfd, p, len) < 0) {
	perror("writetty");
	Exit(1);
    }
}

inline void
rstr(u_char *p, int len)
{

    if (readtty(Camfd, p, len) < 0) {
	perror("readtty");
	Exit(1);
    }
}
